# -*- coding: utf-8 -*-

from django.contrib import admin, messages
from django.core.urlresolvers import reverse
from django.shortcuts import redirect
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe

from jmb.core.utils import models_utils


def clone_car_items(modeladmin, request, queryset):
    done_count = 0
    kw = {
        'kwargs_excludes': [
            'date_create', 'date_last_modify', 'ordering',
            'creator', 'last_modifier',
        ]
    }
    for car in queryset.all():
        done, msg, new_car_item = models_utils.ModelObjectCopier(car, related_parent_name=None, **kw).bulk_copy()
        if done:
            done_count += 1
    messages.info(request, _("Created {} new cars".format(done_count)))
    return redirect(request.META.get('HTTP_REFERER'))
clone_car_items.short_description = _("Clone car items")


def refresh_car_items_slug(modeladmin, request, queryset):
    from publieditor.content.models import Car
    done_count = 0
    for car_item in queryset:
        slug = slugify(car_item.version)
        new_slug = slug
        cont = 0
        while True:
            if Car.objects.filter(slug=new_slug).exclude(id=car_item.id).count():
                cont += 1
                new_slug = "%s-%s" % (slug, cont)
            else:
                break
        car_item.slug = new_slug
        car_item.save()
    messages.info(request, _("Items slug refreshed"))
    return redirect(request.META.get('HTTP_REFERER'))
refresh_car_items_slug.short_description = _("Refresh items slug")
