# -*- coding: utf-8 -*-

from django.db import transaction
from django.conf import settings
from jmb.core.utils.data_import import XlsFile
from django.utils.translation import ugettext_lazy as _
from django.template.defaultfilters import slugify

from publieditor.content.models import UNRAECarModelSale, Car, Brand, CarModel, Corporate


class UNRAECarModelImport(XlsFile):

    def __init__(self, model, filename=None, file_contents=None, auto=False, sheet_index=0, *args, **kwargs):
        super(UNRAECarModelImport, self).__init__(
            filename=filename, file_contents=file_contents, auto=auto, sheet_index=sheet_index
        )
        self.model = model
        self.year = kwargs['year']
        self.messages = dict()
        self.messages['inserted'] = 0
        self.messages['modified'] = 0
        self.messages['unmodified'] = 0
        self.messages['errors'] = []

    def do(self, row, kw, j):
        # ESEMPIO
        # Row(
        #     Segmento=u'A', Marca=u'ABARTH', Modello=500.0,
        #     GennaioFuoristrada=0.0, GennaioCabrioeSpider=0.0, GennaioBerlina2Vol=140.0,
        #     GennaioCoup=0.0, GennaioBerlina3Vol=0.0, GennaioMultispazio=0.0,
        #     GennaioMonovolume=0.0, GennaioStationWagon=0.0, GennaioCrossover=0.0,
        #     GennaioAltre=0.0, TotaleGennaio=140.0,
        #     FebbraioFuoristrada=0.0, FebbraioCabrioeSpider=0.0, FebbraioBerlina2Vol=162.0,
        #     FebbraioCoup=0.0, FebbraioBerlina3Vol=0.0, FebbraioMultispazio=0.0,
        #     FebbraioMonovolume=0.0, FebbraioStationWagon=0.0, FebbraioCrossover=0.0,
        #     FebbraioAltre=0.0, TotaleFebbraio=162.0,...
        # )
        # row[0]            -> 'A'
        # row.Segmento      -> 'A'

        try:
            brand = row.Marca
            carmodel = row.Modello
            if isinstance(brand, (int, float)):
                brand = ("%.0f" % brand).upper()
            if isinstance(carmodel, (int, float)):
                carmodel = ("%.0f" % carmodel).upper()
        except Exception, e:
            self.messages['errors'].append(
                'riga %s - %s %s: %s' % (j, row.Marca, row.Modello, e)
            )
            return

        if brand.strip() != "" and carmodel.strip() != "":

            designs = (
                'Fuoristrada', 'CabrioeSpider', 'Berlina2Vol', 'Coup',
                'Berlina3Vol', 'Multispazio', 'Monovolume', 'StationWagon',
                'Crossover', 'Altre', 'Totale'
            )
            first_col_num = 3   # prima colonna 0 - la prima colonna che contiene i dati di vendita è la 3
            design_num = len(designs)
            col_num = len(row)  # numero totale di colonne
            month_num = (col_num - first_col_num) / design_num
            for month in range(month_num):
                for num, design in enumerate(designs):
                    if design == "Totale":
                        # il 'Totale' non viene importato - non è un design
                        continue

                    ind = first_col_num + (month * design_num) + num
                    if isinstance(row[ind], (int, float)) and row[ind] > 0:

                        try:
                            unrae_carmodel, created = self.model.objects.get_or_create(
                                brand=brand, carmodel=carmodel, design=design
                            )
                            unrae_carmodel_sale, sale_created = UNRAECarModelSale.objects.get_or_create(
                                unrae_carmodel=unrae_carmodel, year=self.year, month=month+1,
                                defaults={'sold': row[ind]}
                            )
                            if sale_created:
                                self.messages['inserted'] += 1
                            else:
                                if row[ind] != unrae_carmodel_sale.sold:
                                    unrae_carmodel_sale.sold = row[ind]
                                    unrae_carmodel_sale.save()
                                    self.messages['modified'] += 1
                                else:
                                    self.messages['unmodified'] += 1
                        except Exception, e:
                            self.messages['errors'].append(
                                'riga %s - %s %s %s: %s' %(j, row.Marca, row.Modello, design, e)
                            )


class CarImportData(XlsFile):

    fields_map = {
        "status": "status",
        "infocar": "infocar",
        # "get_corporate_id": "corporate_id",
        # "get_corporate_name": "corporate_name",
        # "get_brand_id": "brand_id",
        # "get_brand_name": "brand_name",
        # "get_model_id": "model_id",
        # "get_model_name": "model_name",
        "motor": "motor",
        "version": "version",
        "price": "price",
        "shape": "shape",
        "target": "target",
        "seat": "seat",
        "engine": "engine",
        "layout": "layout",
        "turbo": "turbo",
        "fuel": "fuel",
        "traction": "traction",
        "gear": "gear",
        "agear": "agear",
        "pollution": "pollution",
        "engine_loc": "engine_loc",
        "doors": "doors",
        "luggage": "luggage",
        "luggage_max": "luggage_max",
        "tank": "tank",
        "weight": "weight",
        "length": "length",
        "width": "width",
        "height": "height",
        "displ": "displ",
        "power_cv": "power_cv",
        "power_kw": "power_kw",
        "power_rpm": "power_rpm",
        "torque_max": "torque_max",
        "torque_rpm": "torque_rpm",
        "speed": "speed",
        "acc": "acc",
        "co2": "co2",
        "tow": "tow",
        "cons_mix": "cons_mix",
        "turning_circle": "turning_circle",
        "warranty": "warranty",
        "wheelbase": "wheelbase",
        "kml_mix": "kml_mix",
        "neopatentati": "neopatentati",
        "area_c": "area_c",
        "power_spec": "power_spec",
        "power_weight": "power_weight",
        "proxymity": "proxymity",
        "price_update": "price_update",
        "field_src": "field_src",
        "new_date": "new_date",
        "mod_date": "mod_date",
        "standard_num": "standard_num",
        "slug": "slug",
        "number_of_engines": "number_of_engines",
        "combustion_engine_power_cv": "combustion_engine_power_cv",
        "combustion_engine_power_kw": "combustion_engine_power_kw",
        "combustion_engine_torque_Nm": "combustion_engine_torque_Nm",
        "electric_engine_power_cv": "electric_engine_power_cv",
        "electric_engine_power_kw": "electric_engine_power_kw",
        "electric_engine_torque_Nm": "electric_engine_torque_Nm",
        "total_power_cv": "total_power_cv",
        "total_power_kw": "total_power_kw",
        "total_torque_Nm": "total_torque_Nm",
        "batteries_capacity": "batteries_capacity",
        "usable_capacity": "usable_capacity",
        "on_board_battery_charger": "on_board_battery_charger",
        "average_electric_consumption": "average_electric_consumption",
        "ac_charging_time_at_3_7_kw": "ac_charging_time_at_3_7_kw",
        "ac_charging_time_at_11_kw": "ac_charging_time_at_11_kw",
        "ac_charging_time_at_22_kw": "ac_charging_time_at_22_kw",
        "dc_charging_time_at_150_kw": "dc_charging_time_at_150_kw",
        "electric_autonomy_in_km": "electric_autonomy_in_km",
        "total_autonomy_in_km": "total_autonomy_in_km",
        "top_speed_in_EV": "top_speed_in_EV",
    }
    choice_fields = {
        'status': settings.PUBLIEDITOR_CAR_STATUS_CHOICES,
        'shape': settings.PUBLIEDITOR_CAR_SHAPE_CHOICES,
        'target': settings.PUBLIEDITOR_CAR_TARGET_CHOICES,
        'seat': settings.PUBLIEDITOR_CAR_SEAT_CHOICES,
        'engine': settings.PUBLIEDITOR_CAR_ENGINE_CHOICES,
        'layout': settings.PUBLIEDITOR_CAR_LAYOUT_CHOICES,
        'fuel': settings.PUBLIEDITOR_CAR_FUEL_CHOICES,
        'traction': settings.PUBLIEDITOR_CAR_TRACTION_CHOICES,
        'gear': settings.PUBLIEDITOR_CAR_GEAR_CHOICES,
        'agear': settings.PUBLIEDITOR_CAR_AGEAR_CHOICES,
        'pollution': settings.PUBLIEDITOR_CAR_POLLUTION_CHOICES,
        'engine_loc': settings.PUBLIEDITOR_CAR_ENGINE_LOC_CHOICES,
        'doors': settings.PUBLIEDITOR_CAR_DOORS_CHOICES,
        'field_src': settings.PUBLIEDITOR_CAR_FIELD_SRC_CHOICES,
    }
    brand = None
    model = None

    @transaction.commit_manually
    def do(self, row, kw, j):
        try:
            if self.has_required_fields(row):
                if "slug" not in kw or not kw['slug']:
                    kw["slug"] = slugify(kw["version"])
                kw.update({
                    'slug': self.calculate_slug(kw['slug']),
                    'brand': self.get_brand(row),
                    'model': self.get_model(row),
                    'corporate': self.get_corporate(row),
                })
                kw.update(self.manage_choice_fields(kw))
                model_fields = [field.name for field in Car._meta.fields]
                for key in [key for key in kw.keys() if key not in model_fields]:
                    del kw[key]
                Car.objects.create(**kw)
                transaction.commit()
            else:
                self.messages['errors'].append(_("Dati obbligatori mancanti"))
        except Exception, e:
            print e
            transaction.rollback()
            self.messages['errors'].append(_("Generic error: {}".format(e)))

    @staticmethod
    def has_required_fields(row):
        if hasattr(row, "version"):
            brand_id = int(row.get_brand_id) if hasattr(row, 'get_brand_id') else None
            brand_name = str(row.get_brand_name) if hasattr(row, 'get_brand_name') else None
            model_id = int(row.get_model_id) if hasattr(row, 'get_model_id') else None
            model_name = str(row.get_model_name) if hasattr(row, 'get_model_name') else None
            if (brand_id or brand_name) and (model_id or model_name):
                # Check brand
                brand_filter = {'id': brand_id} if brand_id else {'name__iexact': brand_name}
                brand_exists = bool(Brand.objects.filter(**brand_filter).count())
                # Check model
                model_filter = {'id': model_id} if model_id else {'name__iexact': model_name}
                model_exists = bool(CarModel.objects.filter(**model_filter).count())
                if brand_exists and model_exists:
                    return True
        return False

    @staticmethod
    def get_brand(row):
        brand_id = int(row.get_brand_id) if hasattr(row, 'get_brand_id') else None
        brand_name = str(row.get_brand_name) if hasattr(row, 'get_brand_name') else None
        brand_filter = {'id': brand_id} if brand_id else {'name__iexact': brand_name}
        return Brand.objects.filter(**brand_filter)[0]

    @staticmethod
    def get_model(row):
        model_id = int(row.get_model_id) if hasattr(row, 'get_model_id') else None
        model_name = str(row.get_model_name) if hasattr(row, 'get_model_name') else None
        model_filter = {'id': model_id} if model_id else {'name__iexact': model_name}
        return CarModel.objects.filter(**model_filter)[0]

    @staticmethod
    def get_corporate(row):
        corporate_id = int(row.get_corporate_id) if \
            hasattr(row, 'get_corporate_id') and row.get_corporate_id else None
        corporate_name = str(row.get_corporate_name) if \
            hasattr(row, 'get_corporate_name') and row.get_corporate_name  else None
        if corporate_id or corporate_name:
            corporate_filter = {'id': corporate_id} if corporate_id else {'name__iexact': corporate_name}
            return Corporate.objects.filter(**corporate_filter)[0]
        return None

    @staticmethod
    def calculate_slug(slug):
        new_slug = slug
        found_elements, counter = Car.objects.filter(slug__iexact=new_slug).count(), 1
        while found_elements:
            new_slug = "{}-{}".format(slug, counter)
            found_elements = Car.objects.filter(slug__iexact=new_slug).count()
            counter += 1
        return new_slug

    def manage_choice_fields(self, kw):
        new_values = dict()
        for key, enabled_values in self.choice_fields.iteritems():
            if key in kw and kw[key]:
                if key in ["seat", "doors"]:
                    try:
                        value = "{}".format(int(kw[key]))
                    except Exception, e:
                        value = "{}".format(kw[key])
                else:
                    value = kw[key]
                new_values[key] = None if value not in [k for k, v in enabled_values] else value
        return new_values

