# -*- coding: utf-8 -*-

import json
import urllib2
from django import template
from django.db.models import Q
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.core.exceptions import ObjectDoesNotExist
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import redirect
from django.views.generic import ListView
from django.conf import settings

from jmb.core.views.import_data import ImportDataView
from publieditor.content.models import (
    CarModel, UNRAECarModel, Car, Content, MediaPhoto, MediaVideo, MediaTable,
    TranslationContent, Issue, Stream
)
from publieditor.content.admin_forms import (
    SetEquipmentForm, ImportUNRAEForm, InsertEquipmentForm,
    GetPhotosForm, GetVideosForm, GetTablesForm, CarImportDataForm
)
from publieditor.content.admin_import import UNRAECarModelImport, CarImportData
from publieditor.universe import feeder
from jmb.core.views.odt import PdfTemplateView


def set_equipment(request, carmodel_id):
    opts = CarModel._meta
    app_label = opts.app_label
    has_perm = request.user.has_perm(opts.app_label + '.' + opts.get_change_permission())
    template = 'admin/%s/%s/equipment.html' % (
        app_label, opts.object_name.lower())

    if request.method == 'POST':
        form = SetEquipmentForm(request, carmodel_id=carmodel_id)
        form.save(request.POST)
    else:
        form = SetEquipmentForm(request, carmodel_id=carmodel_id)
        form.load()

    context = {
        'app_label': app_label,
        'opts': opts,
        'form': form,
        'app_label': opts.app_label,
        'has_change_permission': has_perm
    }

    return render_to_response(
        template,
        context, context_instance=RequestContext(request)
    )


def model_sync(request, **kwargs):
    from publieditor.content.utils import set_unraecarmodel_carmodels
    set_unraecarmodel_carmodels()
    messages.info(request, _('Model sync completed'))
    return redirect(reverse("admin:content_unraecarmodel_changelist"))


def update_sales_data(request, **kwargs):
    from publieditor.content.utils import set_model_sale_list
    set_model_sale_list()
    messages.info(request, _('Sales data update done'))
    return redirect(reverse("admin:content_unraecarmodel_changelist"))


def import_unrae(request, **kwargs):
    model = UNRAECarModel
    app_label = _('Content')
    verbose_name = _('UNRAE Models')

    template_name = 'import_unrae.html'
    breadcrumbs = (
        (_('Home'), '/admin/'),
        (_(app_label), '/admin/content/'),
        (_(verbose_name), '/admin/content/unraecarmodel/'),
    )

    if request.method == 'POST':
        form = ImportUNRAEForm(request.POST, request.FILES)
        if form.is_valid():
            file_source = form.cleaned_data['import_file']
            year = form.cleaned_data['year']
            kwargs.update({'year': year})
            import_data = UNRAECarModelImport(
                file_contents=file_source.read(), auto=True, model=model, **kwargs
            )
            import_data.read()
            if import_data.messages['inserted'] > 0:
                messages.success(request, "%s righe inserite correttamente" % import_data.messages['inserted'])
            if import_data.messages['modified'] > 0:
                messages.warning(request, "%s righe modificate correttamente" % import_data.messages['modified'])
            if import_data.messages['unmodified'] > 0:
                messages.info(request, "%s righe non modificate" % import_data.messages['unmodified'])
            if import_data.messages['errors']:
                for error in import_data.messages['errors']:
                    messages.error(request, error)
        else:
            messages.error(request, "Correggi l'errore qui sotto")
    else:
        form = ImportUNRAEForm()

    context = {
        'title': _('Import %s') % verbose_name,
        # 'is_popup': "_popup" in request.REQUEST,
        'app_label': app_label,
        'breadcrumbs': breadcrumbs,
        'import_form': form
    }
    context_instance = template.RequestContext(request)
    return render_to_response(
        [
            "admin/content/unraecarmodel/%s" % template_name,
            "admin/content/%s" % template_name,
            "admin/%s" % template_name
        ],
        context,
        context_instance=context_instance
    )


def insert_equipments(request, carmodel_id):
    opts = CarModel._meta
    app_label = opts.app_label
    has_perm = request.user.has_perm(opts.app_label + '.' + opts.get_change_permission())
    context_instance = template.RequestContext(request)
    template_name = 'admin/%s/%s/insert_equipments.html' % (app_label, opts.object_name.lower())
    
    if request.method == 'POST':
        form = InsertEquipmentForm(request.POST, carmodel_id=carmodel_id)
        if form.is_valid():
            form.save()
            message = "Inseriti %s di %s importati" % (form.total_insert, form.total_import)
            if form.total_insert == form.total_import:
                messages.success(request, message)
            elif form.total_insert == 0:
                messages.error(request, message)
            else:
                messages.warning(request, message)
            return HttpResponse("<html><script>parent.iframe_callback()</script></html>")
    else:
        form = InsertEquipmentForm(carmodel_id=carmodel_id)
        
    context = {
        'is_popup': True,
        'app_label': app_label,
        'opts': opts,
        'form': form,
        'has_change_permission': has_perm
    }
    return render_to_response(template_name,
        context,
        context_instance=context_instance
    )


def get_photos(request, news_id):
    # TODO : decidere se integrare anche i permessi successivamente.
    # opts = MediaPhoto._meta
    # app_label = opts.app_label
    # has_perm = request.user.has_perm(opts.app_label + '.' + opts.get_change_permission())

    context_instance = template.RequestContext(request)
    template_name = 'admin/content/content/get_photos.html'

    try:
        news = Content.objects.get(id=news_id)
    except ObjectDoesNotExist:
        message = "Non esiste una news con id %s" % news_id
        messages.error(request, message)
        return HttpResponseRedirect(reverse('admin:content_content_changelist'))

    if request.method == 'POST':
        form = GetPhotosForm(request.POST)

        if form.is_valid():
            saved_photos = MediaPhoto.objects
            old_photos = saved_photos.filter(content=news)

            # creazione dizionario vecchie foto
            old_photos_dict = {}
            for photo in old_photos:
                key = str(photo.photo_id)
                old_photos_dict[key] = photo

            # creazione dizionario nuove foto
            new_photos_dict = {}
            for key in request.POST.keys():
                if key.startswith("photoId__"):
                    # split key to get id
                    dict_key = key.split("__")[1].split("_")[1]
                    new_photos_dict[dict_key] = request.POST[key]

            old_photos_set = set(old_photos_dict)
            new_photos_set = set(new_photos_dict)
            intersection = old_photos_set & new_photos_set

            photos_to_delete = list(old_photos_set - intersection)
            photos_to_add = list(new_photos_set - intersection)
            already_saved = list(old_photos_set)

            # elimina le foto rimosse
            for del_id in photos_to_delete:
                try:
                    MediaPhoto.objects.get(content__id=news_id, photo_id=del_id).delete()
                except ObjectDoesNotExist:
                    pass

            # crea array guida per ordine
            order_array = request.POST.get("media_order", "").split(",")

            # add new photos
            order_index = 1
            for ord_id in order_array:
                if ord_id in already_saved:
                    try:
                        photo = old_photos.get(photo_id=ord_id)
                        photo.ordering = order_index
                        photo.save()
                        order_index += 1
                    except Exception,ex:
                        pass

                elif ord_id in photos_to_add:
                    if "photoId__" + "000_" + ord_id in request.POST:
                        try:
                            raw_post_str_obj = request.POST["photoId__" + "000_" + ord_id]
                            post_str_obj = urllib2.unquote(raw_post_str_obj).encode('utf-8').decode('utf-8')
                            post_dict_obj = json.loads(post_str_obj)

                            new_photo, created = MediaPhoto.objects.get_or_create(
                                content_id=news.id,
                                photo_id=post_dict_obj["id"],
                            )

                            if created:
                                new_photo.title = post_dict_obj["title"] if ("title" in post_dict_obj) else ''
                                new_photo.description = post_dict_obj["description"] if ("description" in post_dict_obj) else ''
                                new_photo.width = post_dict_obj["width"] if "width" in post_dict_obj else None
                                new_photo.height = post_dict_obj["height"] if "height" in post_dict_obj else None
                                new_photo.weigth = post_dict_obj["contentSize"] if "contentSize" in post_dict_obj else None
                                new_photo.mime_type = post_dict_obj["contentType"] if "contentType" in post_dict_obj else ''
                                new_photo.has4_3 = post_dict_obj["has4_3"] if "has4_3" in post_dict_obj else False
                                new_photo.has16_9 = post_dict_obj["has16_9"] if "has16_9" in post_dict_obj else False
                                if "galleryId" in post_dict_obj and created:
                                    new_photo.gallery_id = post_dict_obj["galleryId"] if "galleryId" in post_dict_obj else ''
                                new_photo.ordering = order_index
                                new_photo.save()
                                order_index += 1
                        except Exception, ex:
                            pass
    else:
        form = GetPhotosForm()

    context = {
        'form': form,
        'news': news,
    }
    return render_to_response(
        template_name,
        context,
        context_instance=context_instance
    )


def get_videos(request, news_id):
    # TODO : decidere se integrare anche i permessi successivamente.
    # opts = MediaVideo._meta
    # app_label = opts.app_label
    # has_perm = request.user.has_perm(opts.app_label + '.' + opts.get_change_permission())

    context_instance = template.RequestContext(request)
    template_name = 'admin/content/content/get_videos.html'

    try:
        news = Content.objects.get(id=news_id)
    except ObjectDoesNotExist:
        message = "Non esiste una news con id %s" % news_id
        messages.error(request, message)
        return HttpResponseRedirect(reverse('admin:content_content_changelist'))

    if request.method == 'POST':
        form = GetVideosForm(request.POST)

        if form.is_valid():
            saved_videos = MediaVideo.objects
            old_videos = saved_videos.filter(content=news)

            # creazione dizionario vecchie foto
            old_videos_dict = {}
            for video in old_videos:
                key = str(video.video_id)
                old_videos_dict[key] = video

            # creazione dizionario nuove foto
            new_videos_dict = {}
            for key in request.POST.keys():
                if key.startswith("videoId__"):
                    # split key to get id
                    dict_key = key.split("__")[1].split("_")[1]
                    new_videos_dict[dict_key] = request.POST[key]

            old_videos_set = set(old_videos_dict)
            new_videos_set = set(new_videos_dict)
            intersection = old_videos_set & new_videos_set

            videos_to_delete = list(old_videos_set - intersection)
            videos_to_add = list(new_videos_set - intersection)
            already_saved = list(old_videos_set)

            # elimina i video rimossi
            for del_id in videos_to_delete:
                try:
                    MediaVideo.objects.get(content__id=news_id, video_id=del_id).delete()
                except ObjectDoesNotExist:
                    pass

            # crea array guida per ordine
            order_array = request.POST.get("media_order", "").split(",")

            # add new videos
            order_index = 1
            for ord_id in order_array:
                if ord_id in already_saved:
                    try:
                        video = old_videos.get(video_id=ord_id)
                        video.ordering = order_index
                        video.save()
                        order_index += 1
                    except Exception,ex:
                        pass

                elif ord_id in videos_to_add:
                    if "videoId__" + "000_" + ord_id in request.POST:
                        try:
                            raw_post_str_obj = request.POST["videoId__" + "000_" + ord_id]
                            post_str_obj = urllib2.unquote(raw_post_str_obj).encode('utf-8').decode('utf-8')
                            post_dict_obj = json.loads(post_str_obj)

                            new_video, created = MediaVideo.objects.get_or_create(
                                content_id=news.id,
                                video_id=post_dict_obj["id"],
                            )
                            if created:
                                new_video.title=post_dict_obj["title"] if ("title" in post_dict_obj) else ''
                                new_video.description=post_dict_obj["description"] if ("description" in post_dict_obj) else ''
                                new_video.width=post_dict_obj["width"] if "width" in post_dict_obj else None
                                new_video.height=post_dict_obj["height"] if "height" in post_dict_obj else None
                                new_video.weigth=post_dict_obj["contentSize"] if "contentSize" in post_dict_obj else None
                                new_video.mime_type=post_dict_obj["contentType"] if "contentType" in post_dict_obj else ''
                                new_video.video_url=post_dict_obj["videoUrl"] if "videoUrl" in post_dict_obj else ''
                                new_video.ordering = order_index
                                new_video.save()
                                order_index += 1
                        except Exception, ex:
                            pass
    else:
        form = GetVideosForm()

    context = {
        'form': form,
        'news': news,
    }
    return render_to_response(
        template_name,
        context,
        context_instance=context_instance
    )


def get_tables(request, news_id):
    # TODO : decidere se integrare anche i permessi successivamente.
    # opts = MediaTable._meta
    # app_label = opts.app_label
    # has_perm = request.user.has_perm(opts.app_label + '.' + opts.get_change_permission())

    context_instance = template.RequestContext(request)
    template_name = 'admin/content/content/get_tables.html'

    try:
        news = Content.objects.get(id=news_id)
    except ObjectDoesNotExist:
        message = "Non esiste una news con id %s" % news_id
        messages.error(request, message)
        return HttpResponseRedirect(reverse('admin:content_content_changelist'))
    if request.method == 'POST':
        form = GetTablesForm(request.POST)

        if form.is_valid():
            saved_tables = MediaTable.objects
            old_tables = saved_tables.filter(content=news)

            # creazione dizionario vecchie foto
            old_tables_dict = {}
            for table in old_tables:
                key = str(table.table_id)
                old_tables_dict[key] = table

            # creazione dizionario nuove foto
            new_tables_dict = {}
            for key in request.POST.keys():
                if key.startswith("tableId__"):
                    # split key to get id
                    dict_key = key.split("__")[1].split("_")[1]
                    new_tables_dict[dict_key] = request.POST[key]

            old_tables_set = set(old_tables_dict)
            new_tables_set = set(new_tables_dict)
            intersection = old_tables_set & new_tables_set

            tables_to_delete = list(old_tables_set - intersection)
            tables_to_add = list(new_tables_set - intersection)
            already_saved = list(old_tables_set)

            # elimina le foto rimosse
            for del_id in tables_to_delete:
                try:
                    MediaTable.objects.get(content__id=news_id, table_id=del_id).delete()
                except ObjectDoesNotExist:
                    pass

            # crea array guida per ordine
            order_array = request.POST.get("media_order", "").split(",")

            # add new tables
            order_index = 1
            for ord_id in order_array:
                if ord_id in already_saved:
                    try:
                        table = old_tables.get(table_id=ord_id)
                        table.ordering = order_index
                        table.save()
                        order_index += 1
                    except Exception,ex:
                        pass

                elif ord_id in tables_to_add:
                    if "tableId__" + "000_" + ord_id in request.POST:
                        try:
                            raw_post_str_obj = request.POST["tableId__" + "000_" + ord_id]
                            post_str_obj = urllib2.unquote(raw_post_str_obj).encode('utf-8').decode('utf-8')
                            post_dict_obj = json.loads(post_str_obj)

                            new_table, created = MediaTable.objects.get_or_create(
                                content_id=news.id,
                                table_id=post_dict_obj["id"],
                            )
                            if created:
                                new_table.title=post_dict_obj["title"] if ("title" in post_dict_obj) else ''
                                new_table.description=post_dict_obj["description"] if ("description" in post_dict_obj) else ''
                                new_table.width=post_dict_obj["width"] if "width" in post_dict_obj else None
                                new_table.height=post_dict_obj["height"] if "height" in post_dict_obj else None
                                new_table.weigth=post_dict_obj["contentSize"] if "contentSize" in post_dict_obj else None
                                new_table.mime_type=post_dict_obj["contentType"] if "contentType" in post_dict_obj else ''
                                new_table.ordering = order_index
                                new_table.save()
                                order_index += 1
                        except Exception, ex:
                            pass
    else:
        form = GetTablesForm()

    context = {
        'form': form,
        'news': news,
    }
    return render_to_response(
        template_name,
        context,
        context_instance=context_instance
    )


class OutputView(feeder.GenericPDFFeed):

    template_name = 'universe/feeds/bosch_en.odt'

    def get_news(self, issue_id):
        news = Content.objects.filter(issue_id=issue_id).order_by('ordering', 'id')
        return news, news.count()

    def get_context_data(self, **kwargs):

        issue_id = self.kwargs['issue_id']
        issue = Issue.objects.get(pk=issue_id)
        news, count = self.get_news(issue_id)

        issue_date = issue.date_content_ordinal()
        ## add a counter
        for j, news_item in enumerate(news):
            news_item.j = j+1

        #self.template_name = universe.template_name or self.template_name
        return super(PdfTemplateView, self).get_context_data(
            pagesize="A4",
            news=news,
            issue_date=issue_date,
            **kwargs
        )


class OutputViewHtml(ListView):
    model = TranslationContent
    template_name = 'admin/content/issue/output_html.html'

    def get_queryset(self):
        issue = get_object_or_404(Issue, id=self.kwargs['issue_id'])
        queryset = super(OutputViewHtml, self).get_queryset().filter(
            content__issue=issue
        )
        if issue.stream:
            if issue.stream.name == "AutolinkNews":
                autolinknews_stream = issue.stream
                autolink_stream = Stream.objects.get(name="Autolink")
                queryset = queryset.filter(
                    Q(content__stream=autolinknews_stream) | Q(content__stream=autolink_stream),
                    language=issue.stream.language
                )
            else:
                queryset = queryset.filter(
                    content__stream=issue.stream,
                    language=issue.stream.language
                )
        return queryset

    def get_context_data(self, **kwargs):
        """
        Get the context for this view.
        """
        issue = get_object_or_404(Issue, id=self.kwargs['issue_id'])
        context = {'issue': issue}
        context.update(kwargs)
        super_result = super(OutputViewHtml, self).get_context_data(**context)
        return super_result


##################################################################################################
# Import views
##################################################################################################
class CarImportView(ImportDataView):
    template_name = 'admin/content/car/import.html'
    model = Car
    import_class = CarImportData
    form_class = CarImportDataForm

    def get_context_data(self, **kwargs):
        context = super(CarImportView, self).get_context_data(**kwargs)
        opts = self.model._meta
        context['opts'] = opts
        context['app_label'] = opts.app_label
        context['PUBLIEDITOR_CAR_STATUS_CHOICES'] = self.get_choice_keys(settings.PUBLIEDITOR_CAR_STATUS_CHOICES)
        context['PUBLIEDITOR_CAR_SHAPE_CHOICES'] = self.get_choice_keys(settings.PUBLIEDITOR_CAR_SHAPE_CHOICES)
        context['PUBLIEDITOR_CAR_TARGET_CHOICES'] = self.get_choice_keys(settings.PUBLIEDITOR_CAR_TARGET_CHOICES)
        context['PUBLIEDITOR_CAR_SEAT_CHOICES'] = self.get_choice_keys(settings.PUBLIEDITOR_CAR_SEAT_CHOICES)
        context['PUBLIEDITOR_CAR_ENGINE_CHOICES'] = self.get_choice_keys(settings.PUBLIEDITOR_CAR_ENGINE_CHOICES)
        context['PUBLIEDITOR_CAR_LAYOUT_CHOICES'] = self.get_choice_keys(settings.PUBLIEDITOR_CAR_LAYOUT_CHOICES)
        context['PUBLIEDITOR_CAR_FUEL_CHOICES'] = self.get_choice_keys(settings.PUBLIEDITOR_CAR_FUEL_CHOICES)
        context['PUBLIEDITOR_CAR_TRACTION_CHOICES'] = self.get_choice_keys(settings.PUBLIEDITOR_CAR_TRACTION_CHOICES)
        context['PUBLIEDITOR_CAR_GEAR_CHOICES'] = self.get_choice_keys(settings.PUBLIEDITOR_CAR_GEAR_CHOICES)
        context['PUBLIEDITOR_CAR_AGEAR_CHOICES'] = self.get_choice_keys(settings.PUBLIEDITOR_CAR_AGEAR_CHOICES)
        context['PUBLIEDITOR_CAR_POLLUTION_CHOICES'] = self.get_choice_keys(settings.PUBLIEDITOR_CAR_POLLUTION_CHOICES)
        context['PUBLIEDITOR_CAR_ENGINE_LOC_CHOICES'] = self.get_choice_keys(settings.PUBLIEDITOR_CAR_ENGINE_LOC_CHOICES)
        context['PUBLIEDITOR_CAR_DOORS_CHOICES'] = self.get_choice_keys(settings.PUBLIEDITOR_CAR_DOORS_CHOICES)
        context['PUBLIEDITOR_CAR_FIELD_SRC_CHOICES'] = self.get_choice_keys(settings.PUBLIEDITOR_CAR_FIELD_SRC_CHOICES)
        return context

    @staticmethod
    def get_choice_keys(choices):
        return ", ".join([key for key, value in choices])

    @property
    def redirect_url(self):
        return reverse('admin:content_car_changelist')
