# -*- coding: utf-8 -*-

from django.utils import simplejson

from dajax.core import Dajax
from dajaxice.decorators import dajaxice_register
from django.template.defaultfilters import slugify

from publieditor.content.models import Brand, CarModel, Car, TranslationContent


@dajaxice_register
def on_select_corporate(request, *args, **kwargs):
    dajax = Dajax()

    options = "<option value='' selected='selected'>---------</option>"
    dajax.assign('#id_brand', 'innerHTML', options)
    dajax.assign('#id_model', 'innerHTML', options)
    dajax.assign('#id_car', 'innerHTML', options)

    id_corporate = kwargs.get('id_corporate', None)
    if not id_corporate:
        return dajax.json()

    brands = Brand.objects.filter(corporate_id=id_corporate).distinct()
    for brand in brands:
        options += "<option value='%s'>%s</option>" % (
            brand.id, brand.name
        )
    dajax.assign('#id_brand', 'innerHTML', options)
    return dajax.json()

@dajaxice_register
def on_select_corporate_news(request, *args, **kwargs):
    dajax = Dajax()

    options = "<option value='' selected='selected'>---------</option>"
    dajax.assign('#id_brand1', 'innerHTML', options)
    dajax.assign('#id_brand2', 'innerHTML', options)
    dajax.assign('#id_model', 'innerHTML', options)
    dajax.assign('#id_car', 'innerHTML', options)

    id_corporate = kwargs.get('id_corporate', None)
    if not id_corporate:
        return dajax.json()

    brands = Brand.objects.filter(corporate_id=id_corporate).distinct()
    for brand in brands:
        options += "<option value='%s'>%s</option>" % (
            brand.id, brand.name
        )
    dajax.assign('#id_brand1', 'innerHTML', options)
    dajax.assign('#id_brand2', 'innerHTML', options)
    return dajax.json()

@dajaxice_register
def on_select_brand(request, *args, **kwargs):
    dajax = Dajax()

    options = "<option value='' selected='selected'>---------</option>"
    dajax.assign('#id_model', 'innerHTML', options)
    dajax.assign('#id_car', 'innerHTML', options)

    id_corporate = kwargs.get('id_corporate', None)
    id_brand = kwargs.get('id_brand', None)
    if not id_brand:
        return dajax.json()

    models = CarModel.objects.filter(brand_id=id_brand)
    if id_corporate:
        models = models.filter(brand__corporate_id=id_corporate)
    for model in models:
        options += "<option value='%s'>%s</option>" % (
            model.id, model.name
        )
    dajax.assign('#id_model', 'innerHTML', options)
    return dajax.json()


@dajaxice_register
def on_select_model(request, *args, **kwargs):
    dajax = Dajax()

    options = "<option value='' selected='selected'>---------</option>"
    dajax.assign('#id_car', 'innerHTML', options)

    id_corporate = kwargs.get('id_corporate', None)
    id_brand = kwargs.get('id_brand', None)
    id_model = kwargs.get('id_model', None)
    if not id_model:
        return dajax.json()

    cars = Car.objects.filter(model_id=id_model)
    if id_brand:
        cars = cars.filter(model__brand_id=id_brand)
    if id_corporate:
        cars = cars.filter(model__brand__corporate=id_corporate)
    for car in cars:
        options += "<option value='%s'>%s</option>" % (
            car.id, car.version
        )
    dajax.assign('#id_car', 'innerHTML', options)
    return dajax.json()


@dajaxice_register
def on_click_btn_make_slug(request, *args, **kwargs):
    dajax = Dajax()
    html_id = kwargs.get('html_id', None)
    content_translation_id = kwargs.get('content_translation_id', None)
    title = kwargs.get('title', None)
    if content_translation_id and title:
        translation_slug = slugify(title)
        new_slug = translation_slug
        cont = 0
        while True:
            if TranslationContent.objects.filter(slug=new_slug).exclude(id=content_translation_id).count():
                cont += 1
                new_slug = "%s-%s" % (translation_slug, cont)
            else:
                break
        dajax.assign('#id_translations-%s-slug' % html_id, 'value', new_slug)
    return dajax.json()


@dajaxice_register
def on_click_car_make_slug(request, *args, **kwargs):
    dajax = Dajax()
    car_id = kwargs.get('car_id', None)
    car_id = car_id if car_id.isdigit() else None
    car_version = kwargs.get('car_version', None)
    if car_version:
        slug = slugify(car_version)
        new_slug = slug
        cont = 0
        while True:
            query = Car.objects.filter(slug=new_slug)
            if car_id:
                query = query.exclude(id=car_id)
            if query.count():
                cont += 1
                new_slug = "%s-%s" % (slug, cont)
            else:
                break
        dajax.assign('#id_slug', 'value', new_slug)
    return dajax.json()