# -*- coding: utf-8 -*-
import autocomplete_light
import json

from django.http import HttpResponse
from django.utils.translation import ugettext_lazy as _

from jmb.core.utils.autocomplete import DynamicAutocompleteModelBase
from jmb.tag.models import Tag

from publieditor.content.models import Car, CarModel, Corporate, Brand, Issue, Content, Kind


class CorporateAutocomplete(autocomplete_light.AutocompleteModelBase):
    model = Corporate
    choices = Corporate.objects
    search_fields = ('name', 'nick',)
    autocomplete_js_attributes = {
        'placeholder': _('Corporate'),
        'minimum_characters': 0,
    }


class BrandAutocompleteBase(autocomplete_light.AutocompleteModelBase):
    model = Brand
    search_fields = ('name', 'nick',)
    autocomplete_js_attributes = {'placeholder': _('Brand')}
    choices = Brand.objects


class BrandAutocompleteDynamic(DynamicAutocompleteModelBase):
    model = Brand
    choices = Brand.objects
    search_fields = ('name', 'nick',)
    autocomplete_js_attributes = {
        'placeholder': _('Brand'),
        'minimum_characters': 0,
    }
    filters = {
        'corporates': '#id_corporate'
    }


class BrandAutocomplete(DynamicAutocompleteModelBase):
    model = Brand
    search_fields = ('name', 'nick',)
    autocomplete_js_attributes = {'placeholder': _('Brand')}
    choices = Brand.objects
    filters = {
        'corporates': '#id_car_details-x-corporate'
    }


class CarModelAutocompleteDynamic(DynamicAutocompleteModelBase):
    model = CarModel
    choices = CarModel.objects
    search_fields = ('name',)
    autocomplete_js_attributes = {
        'placeholder': _('Car model'),
        'minimum_characters': 0,
    }
    filters = {
        'brand': '#id_brand',
    }


class CarModelAutocompleteBase(autocomplete_light.AutocompleteModelBase):
    model = CarModel
    search_fields = ('name',)
    autocomplete_js_attributes = {'placeholder': 'Model'}
    choices = CarModel.objects


class CarModelAutocomplete(DynamicAutocompleteModelBase):
    model = CarModel
    search_fields = ('name',)
    autocomplete_js_attributes = {'placeholder': 'Model'}
    choices = CarModel.objects
    filters = {
        'brand_id': '#id_car_details-x-brand'
    }


class CarAutocomplete(DynamicAutocompleteModelBase):
    model = Car
    search_fields = ('version',)
    autocomplete_js_attributes = {'placeholder': 'Car'}
    choices = Car.objects
    filters = {
        'corporate_id': '#id_car_details-x-corporate',
        'brand_id': '#id_car_details-x-brand',
        'model_id': '#id_car_details-x-model'
    }


class IssueAutocomplete(autocomplete_light.AutocompleteModelBase):
    model = Issue
    search_fields = ('number', 'id',)
    autocomplete_js_attributes = {'placeholder': 'Issue'}
    choices = Issue.objects


class ParentAutocomplete(autocomplete_light.AutocompleteModelBase):
    model = Content
    search_fields = ('name', 'id')
    autocomplete_js_attributes = {'placeholder': 'Parent'}
    choices = Content.objects


class ParentKindAutocomplete(autocomplete_light.AutocompleteModelBase):
    model = Kind
    search_fields = ('name', 'id')
    autocomplete_js_attributes = {'placeholder': 'Kind Children'}
    choices = Kind.objects


class NewTagAutocomplete(autocomplete_light.AutocompleteModelTemplate):
    search_fields = ('name',)
    autocomplete_js_attributes = {'placeholder': 'Tag'}
    choices = Tag.objects.all()
    model = Tag
    widget_template = "/autocomplete_light/tagwidget.html"

    def autocomplete_html(self):
        choices = self.choices_for_request()
        results = []
        for choice in choices:
            results.append({"id": choice.pk, "text": str(choice)})
        return HttpResponse(json.dumps(results), content_type="application/json")


autocomplete_light.register(CorporateAutocomplete, name="CorporateAutocomplete")
autocomplete_light.register(BrandAutocomplete, name="BrandAutocomplete")
autocomplete_light.register(BrandAutocompleteBase, name="BrandAutocompleteBase")
autocomplete_light.register(BrandAutocompleteDynamic, name="BrandAutocompleteDynamic")
autocomplete_light.register(CarModelAutocompleteDynamic, name="CarModelAutocompleteDynamic")
autocomplete_light.register(CarModelAutocomplete, name="CarModelAutocomplete")
autocomplete_light.register(CarModelAutocompleteBase, name="CarModelAutocompleteBase")
autocomplete_light.register(CarAutocomplete, name="CarAutocomplete")
autocomplete_light.register(IssueAutocomplete, name="IssueAutocomplete")
autocomplete_light.register(ParentAutocomplete, name="ParentAutocomplete")
autocomplete_light.register(ParentKindAutocomplete, name="ParentKindAutocomplete")
autocomplete_light.register(NewTagAutocomplete, name="NewTagAutocomplete")
