# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext as _
from django.conf import settings

from cms.models.pluginmodel import CMSPlugin

from jmb.core.db.models import StatusModel

# This line shoudl fix the DeprecationWarning
# http://stackoverflow.com/questions/16682693/django-cms-warning-on-django-mptt
CMSPlugin.tree = CMSPlugin.objects

class TodaySettings(CMSPlugin, StatusModel):
    template_name = models.CharField(
        max_length=100,
        choices=settings.PUBLIEDITOR_CONTENT_CMSPLUGIN_TODAY_TEMPLATES,
    )

    class Meta:
        app_label = "content"

    def __unicode__(self):
        return "%s %s - %s" % (
            ('On - ' if self.status else 'Off - '), self._meta.app_label,
            self.get_template_name_display()
        )


class ContentDetailSettings(CMSPlugin, StatusModel):
    template_name = models.CharField(
        max_length=100,
        choices=settings.PUBLIEDITOR_CONTENT_CMSPLUGIN_DETAIL_TEMPLATES,
    )
    content = models.ForeignKey(
        'Content',
        verbose_name=_("Content")
    )

    class Meta:
        app_label = "content"

    def __unicode__(self):
        return "%s %s - %s" % (
            ('On - ' if self.status else 'Off - '), self._meta.app_label,
            self.get_template_name_display()
        )


class ContentSettings(CMSPlugin, StatusModel):
    title = models.CharField(
        max_length=100,
        null=True, blank=True,
        verbose_name=_("title"),
    )
    slug = models.SlugField(
        max_length=100,
        verbose_name=_("slug")
    )
    template_name = models.CharField(
        max_length=100,
        choices=settings.PUBLIEDITOR_CONTENT_CMSPLUGIN_LIST_TEMPLATES
    )
    show_if_authenticated = models.CharField(
        max_length=100,
        choices=settings.PUBLIEDITOR_CONTENT_CMSPLUGIN_SHOW_IF_AUTHENTICATED,
        verbose_name=_('Show if autenticated'),
        help_text = _("Show plugin if user is authenticated")
    )
    stream = models.ForeignKey(
        'Stream',
        null=True, blank=True,
        verbose_name=_('stream')
    )
    # category_slugs_included = models.CharField(
    #     max_length=100,
    #     blank=True, null=True,
    #     verbose_name=_("Including category slugs"),
    #     help_text=_("Including content category slugs filter")
    # )
    # category_slugs_excluded = models.CharField(
    #     max_length=100,
    #     blank=True, null=True,
    #     verbose_name=_("Excluding category slugs"),
    #     help_text=_("Eexcluding content category slugs filter")
    # )
    content_slugs_included = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_('Including content slugs'),
        help_text=_("Including content slugs filter")
    )
    content_slugs_excluded = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("Excluding slugs"),
        help_text=_("Excluding content slugs filter")
    )
    content_page_slug = models.SlugField(
        max_length=100,
        null=True, blank=True,
        verbose_name=_("Content page slug"),
        help_text=_("Content page slug")
    )
    content_filter = models.CharField(
        max_length=100,
        choices=settings.PUBLIEDITOR_CONTENT_CMSPLUGIN_CONTENT_FILTER
    )
    content_days_included = models.IntegerField(
        blank=True, null=True,
        verbose_name=_("Including last number days"),
    )
    content_days_excluded = models.IntegerField(
        blank=True, null=True,
        verbose_name=_("Excluding last number days"),
    )
    objects_date_range = models.CharField(
        max_length=100,
        choices=settings.PUBLIEDITOR_CONTENT_CMSPLUGIN_OBJECTS_DATE_RANGE
    )
    pub_date_begin = models.DateTimeField(
        blank=True, null=True,
        verbose_name=_('Publication date begin'),
        help_text=_('Content publication date starts'),
    )
    pub_date_end = models.DateTimeField(
        blank=True, null=True,
        verbose_name=_('Publication date end'),
        help_text=_('Content publication date ends'),
    )
    ordering = models.CharField(
        max_length=100,
        default='content__ordering, -content__date_content, title',
        verbose_name=_('Content ordering')
    )
    objects_limit = models.IntegerField(
        blank=True, null=True,
        verbose_name=_("Objects number limit"),
    )
    objects_per_page = models.IntegerField(
        default=10,
        verbose_name=_('Objects per page')
    )
    adjacent_pages = models.IntegerField(
        default=2,
        verbose_name=_('Adjacent pages')
    )

    class Meta:
        app_label = "content"

    def __unicode__(self):
        return "%s: %s %s - %s" % (
            self.id, ('On - ' if self.status else 'Off - '), self._meta.app_label,
            self.get_template_name_display()
        )

    def has_title(self):
        if not self.title:
            return False
        return True if self.title.strip() != "" else False


class ContentSearchSettings(CMSPlugin, StatusModel):
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.PUBLIEDITOR_CONTENT_FORM_CLASSES,
        default="publieditor.content.cms.forms.SearchContentForm",
    )
    content_slug = models.SlugField(
        max_length=100,
        verbose_name=_("Content slug")
    )
    template_name = models.CharField(
        max_length=100,
        choices=settings.PUBLIEDITOR_CONTENT_CMSPLUGIN_SEARCH_TEMPLATES,
        default="content/cms_plugins/search.html"
    )

    class Meta:
        app_label = "content"

    def __unicode__(self):
        return "%s %s - %s" % (
            ('On - ' if self.status else 'Off - '), self._meta.app_label,
            self.get_template_name_display()
        )


class ContentAdvancedSearchSettings(CMSPlugin, StatusModel):
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.PUBLIEDITOR_CONTENT_FORM_CLASSES,
        default="publieditor.content.cms.forms.AdvancedSearchContentForm",
    )
    content_slug = models.SlugField(
        max_length=100,
        verbose_name=_("Content slug")
    )
    template_name = models.CharField(
        max_length=100,
        choices=settings.PUBLIEDITOR_CONTENT_CMSPLUGIN_SEARCH_TEMPLATES,
        default="content/cms_plugins/advanced_search.html"
    )

    class Meta:
        app_label = "content"

    def __unicode__(self):
        return "%s %s - %s" % (
            ('On - ' if self.status else 'Off - '), self._meta.app_label,
            self.get_template_name_display()
        )
