# -*- coding: utf-8 -*-

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

import admin_tools
from admin_tools.dashboard import modules, AppIndexDashboard

from jmb.core.admin import dashboard as jmb_dashboard


class DefaultDashboard(modules.Group):
    def __init__(self, **kwargs):
        kwargs.update({
            'title': _("Content"),
            'display': "tabs",
            'children': [ContentModels(), CarModels(), UNRAEModels(), OtherModels()]
        })
        super(DefaultDashboard, self).__init__(**kwargs)


class ContentDashboard(modules.Group):
    def __init__(self, **kwargs):
        kwargs.update({
            'title': _("Content"),
            'display': "tabs",
            'children': [ContentModels(), CarModels(), UNRAEModels(), OtherModels()]
        })
        super(ContentDashboard, self).__init__(**kwargs)


class ContentModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs.update({
            'title': _("Content Models"),
            'models': (
                'publieditor.content.models.Content',
                # 'publieditor.content.models.TestDrive',
                # 'publieditor.content.models.OpenHouse',
                # 'publieditor.content.models.CrashTest',
                # 'publieditor.content.models.Brochure',
                # 'publieditor.content.models.Catalog',
                # 'publieditor.content.models.Video',
                # 'publieditor.content.models.RoadTest',
                # 'publieditor.content.models.PhotoGallery',
                # 'publieditor.content.models.Gallery',
                # 'publieditor.content.models.Approfondimento',
                # 'publieditor.content.models.Breve',
                # 'publieditor.content.models.News',
            ),
        })
        super(ContentModels, self).__init__(**kwargs)


class CarModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs.update({
            'title': _("Car Models"),
            'models': (
                'publieditor.content.models.Corporate',
                'publieditor.content.models.Brand',
                'publieditor.content.models.CarModel',
                'publieditor.content.models.Car',
                'publieditor.content.models.Hit',
                'publieditor.content.models.SalePeriod',
                'publieditor.content.models.CarModelSale',
                'publieditor.content.models.CarModelEquipment',
                'publieditor.content.models.CarEquipment',
            ),
        })
        super(CarModels, self).__init__(**kwargs)


class UNRAEModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs.update({
            'title': _("UNRAE Models"),
            'models': (
                'publieditor.content.models.UNRAECarModel',
                'publieditor.content.models.UNRAECarModelSale',
            ),
            'extra': [
                {'title': 'Model Import', 'change_url': reverse('admin:content_unraecarmodel_import_data')},
            ]
        })
        super(UNRAEModels, self).__init__(**kwargs)


class OtherModels(modules.ModelList):
    def __init__(self, **kwargs):
        kwargs.update({
            'title': _("Other Models"),
            'models': (
                'publieditor.content.models.Issue',
                'publieditor.content.models.Stream',
                'publieditor.content.models.Layout',
                'publieditor.content.models.Sector',
                'publieditor.content.models.Kind',
            )
        })
        super(OtherModels, self).__init__(**kwargs)


class DefaultAppDashboard(AppIndexDashboard):
    def __init__(self, app_title, models, **kwargs):
        AppIndexDashboard.__init__(self, app_title, models, **kwargs)
        self.columns = 3

    def init_with_context(self, context):
        self.children.append(
            DefaultDashboard(),
        )

# Used in /admin 
jmb_dashboard.register_group(ContentDashboard, 'publieditor.content')
# Used in /admin/Content
admin_tools.dashboard.register(DefaultAppDashboard, 'publieditor.content')
