# coding: utf-8
"""
.. _aziende_attive:

Aziende attive
=============================

Command per utilizzo::

  dj aziende_attive

:-d: variabile Booleana per cancellare il messaggi una volta recuperati
:-l: variabile Numero per limitare il numero di messaggi recuperare
:-v: variabile Booleana per visualizzare i print
:-b, --backup: backup messages in 
"""


import re
import os
import sys
import email
import tempfile
import logging
import datetime
import random
from optparse import make_option

from django.conf import settings
from django.core.mail.message import EmailMessage
from django.core.management.base import BaseCommand
from django.conf import settings
from django.core import mail
from django.db import connection, transaction

from jmb.l10n.models import Country

from jmb.organization.models import Organization, OrganizationType, OrganizationForm, OrganizationAddress, Contact
from jmb.core.utils.data_import import XlsFile,  CsvFile


class Anagrafiche(XlsFile):

    fields_map = dict(
        code='ID_Azienda',
        name='Azienda_Nome',
        note='Azienda_Note',
        vat_number='Azienda_PartIVA',
        ssn='Azienda_PartIVA',
    )
    verbosity = 0
    organization_type_name = "Cliente"
    organization_form_name = "Non definito"

    def do(self, row, kw, j):
        kw["id"] = kw["code"] = int(kw["code"])
        kw['organization_type'] = self.set_organization_type()
        kw['organization_form'] = self.set_organization_form()
        organization = None
        try:
            organization = Organization.objects.get(vat_number=kw['vat_number'])
        except Organization.DoesNotExist:
            if kw["vat_number"] == "":
                kw["vat_number"] = "xx%s" % '{0:09}'.format(random.randint(0, 999999999)).zfill(9)
            organization = Organization(**kw)
            organization.save()
        if organization:
            self.set_address(row, organization)
            self.set_contact(row, organization)

    def set_organization_type(self):
        organization_types = OrganizationType.objects.filter(name=self.organization_type_name)
        if not len(organization_types):
            organization_type = OrganizationType()
            organization_type.category = "cus"
            organization_type.code = self.organization_type_name
            organization_type.name = self.organization_type_name
            organization_type.save()
        else:
            organization_type = organization_types[0]
        return organization_type

    def set_organization_form(self):
        organization_forms = OrganizationForm.objects.filter(name=self.organization_form_name)
        if not len(organization_forms):
            organization_form = OrganizationForm()
            organization_form.code = self.organization_form_name
            organization_form.name = self.organization_form_name
            organization_form.save()
        else:
            organization_form = organization_forms[0]
        return organization_form

    def set_address(self, row, organization):
        try:
            address = OrganizationAddress()
            address.organization = organization
            address.name = row.Azienda_Indirizzo
            address.zip_code = row.Azienda_CAP
            address.locality = row.Azienda_Localit
            address.province = row.Azienda_Provincia[:2]
            address.country = Country.objects.get(name__icontains=row.Azienda_Paese.strip())
            address.save()
        except Exception, e:
            print "Errore in set_address di %s -> %s" % (organization.name, e)

    def set_contact(self, row, organization):
        try:
            if row.Azienda_Contatto.strip() != "":
                contact = Contact()
                contact.organization = organization
                contact.first_name = row.Azienda_Contatto.split(" ")[0]
                contact.last_name = " ".join(row.Azienda_Contatto.split(" ")[1:])
                phone_split_str = row.Azienda_Contatto_Tel.split("-")
                if len(phone_split_str) >= 1:
                    contact.phone = phone_split_str[0]
                if len(phone_split_str) > 1:
                    contact.mobile = row.Azienda_Contatto_Tel.split("-")[1]
                contact.save()
            else:
                print "Impssibile creare contatto per %s -> contatto non definito" % (organization.name)
        except Exception, e:
            print "Errore in set_contact di %s -> %s" % (organization.name, e)


class Command(BaseCommand):
    """Get all messages from configured Host
    """
    help = "Marca come attive le aziende del file xls?"

    option_list = BaseCommand.option_list + (
        make_option('-D', '--debug', action='store_true', default=False,
                    dest='debug_message', help='Debug '),
        make_option('-l', '--limit', type=int, dest='limit', help='Limit Messages', 
                    default=0),
        make_option('-s', '--start', type=int, dest='start', help='Start from message n. START', 
                    default=0),
        make_option('-f', '--file', dest='filename', help='Leggi i dati dal file FILE', default=None),
    )

    output_transaction = True

    def handle(self, *args, **opts):
        limit = opts.get('limit')
        start = opts.get('start')
        if not opts.get('filename'):
            print "Manca il nome del file da importare (opt -f)"
            sys.exit(1)

        Organization.objects.all().delete()
        OrganizationAddress.objects.all().delete()
        Contact.objects.all().delete()

        x = Anagrafiche(opts.get('filename'), auto=True, )
        x.verbosity = opts.get('verbosity')
        x.read(start=start, limit=limit)

        self.set_sequence('organization_organization')

    def set_sequence(self, table):
        cursor = connection.cursor()
        select = "SELECT setval('%s_id_seq', (SELECT MAX(id) FROM %s));" % (table, table)
        cursor.execute(select)
        row = cursor.fetchone()

# Example usage: dj import_aziende_xslx_aziende -f /tmp/Aziende.xlsx
