# coding: utf-8
"""
.. _aziende_attive:

Aziende attive
=============================

Command per utilizzo::

  dj aziende_attive

:-d: variabile Booleana per cancellare il messaggi una volta recuperati
:-l: variabile Numero per limitare il numero di messaggi recuperare
:-v: variabile Booleana per visualizzare i print
:-b, --backup: backup messages in 
"""


import re
import os
import sys
import email
import tempfile
import logging
import datetime
from optparse import make_option

from django.conf import settings
from django.core.mail.message import EmailMessage
from django.core.management.base import BaseCommand
from django.conf import settings
from django.contrib.auth.models import User
from django.core import mail
from django.db import connection, transaction

from jmb.l10n.models import Country

from jmb.organization.models import Organization, OrganizationType, OrganizationForm, OrganizationAddress, Contact
from jmb.core.utils.data_import import XlsFile,  CsvFile


class Anagrafiche(XlsFile):

    fields_map = dict(
        last_name='Contatto_Cognome',
        first_name='Contatto_Nome',
        email='Contatto_Email',
        note='Contatto_Note',
        mobile='Mobile',
        fax='Fax',
    )
    verbosity = 0
    organization_type_name = "Cliente"
    organization_form_name = "Non definito"

    def do(self, row, kw, j):
        organizations = Organization.objects.filter(name__iexact=row.Azienda_Nome.strip())
        if len(organizations) > 0:

            # Creo user django
            last_name = kw['last_name'].strip().lower()[:30]
            first_name = kw['first_name'].strip().lower()[:30]
            if last_name != "" and first_name != "":
                user_name = ("%s_%s" % (last_name, first_name))[:30]
                try:
                    user = User.objects.get(username=user_name)
                except User.DoesNotExist:
                    user = User()
                    user.username = user_name
                    user.email = kw['email']
                    user.first_name = first_name
                    user.last_name = last_name
                    user.set_password("publieditor2014")
                    user.save()

                # Creo record colelgato in jmb.organization.contact
                if not Contact.objects.filter(user=user).count():
                    kw['organization'] = organizations[0]
                    kw['user'] = user
                    kw['status'] = 0 if row.Stato.strip() == "ELIMINATO" else 1
                    contact = Contact(**kw)
                    contact.save()
                    print "Contact %s %s created for %s" % (
                        kw['first_name'],
                        kw['last_name'],
                        kw['organization'].name,
                    )

class Command(BaseCommand):
    """Get all messages from configured Host
    """
    help = "Marca come attive le aziende del file xls?"

    option_list = BaseCommand.option_list + (
        make_option('-D', '--debug', action='store_true', default=False,
                    dest='debug_message', help='Debug '),
        make_option('-l', '--limit', type=int, dest='limit', help='Limit Messages', 
                    default=0),
        make_option('-s', '--start', type=int, dest='start', help='Start from message n. START', 
                    default=0),
        make_option('-f', '--file', dest='filename', help='Leggi i dati dal file FILE', default=None),
    )

    output_transaction = True

    def handle(self, *args, **opts):
        limit = opts.get('limit')
        start = opts.get('start')
        if not opts.get('filename'):
            print "Manca il nome del file da importare (opt -f)"
            sys.exit(1)

        Contact.objects.all().delete()

        x = Anagrafiche(opts.get('filename'), auto=True, )
        x.verbosity = opts.get('verbosity')
        x.read(start=start, limit=limit)

        self.set_sequence('organization_contact')

    def set_sequence(self, table):
        cursor = connection.cursor()
        select = "SELECT setval('%s_id_seq', (SELECT MAX(id) FROM %s));" % (table, table)
        cursor.execute(select)
        row = cursor.fetchone()

# Example usage: dj import_aziende_xslx_contatti -f /tmp/Contatti.xlsx
