# -*- coding: utf-8 -*-
from django import template
from django.template import defaulttags
from django.contrib.sites.models import Site

from publieditor.content.models import TranslationContent, UserContentSettings, Tooltip

register = template.Library()

@register.filter(name='get_content_is_read')
def get_content_is_read(translation_content, user):
    user_content_settings = UserContentSettings.objects.filter(
        translation_content=translation_content,
        user=user
    )
    if user_content_settings:
        user_content_setting = user_content_settings[0]
        return user_content_setting.is_read
    return False

@register.filter(name='get_content_is_favorite')
def get_content_is_favorite(translation_content, user):
    user_content_settings = UserContentSettings.objects.filter(
        translation_content=translation_content,
        user=user
    )
    if user_content_settings:
        user_content_setting = user_content_settings[0]
        return user_content_setting.is_favorite
    return False


@register.filter(name='get_content_is_removed')
def get_content_is_removed(translation_content, user):
    user_content_settings = UserContentSettings.objects.filter(
        translation_content=translation_content,
        user=user
    )
    if user_content_settings:
        user_content_setting = user_content_settings[0]
        return user_content_setting.is_removed
    return False


@register.filter(name='get_range')
def get_range(value):
    value = value or 0
    return range( value )


## EX USO : {% get_tooltip_description "IT" "scarica-pdf" %}
## attenzione all'ordine dei parametri
@register.simple_tag(name='get_tooltip_description')
def get_tooltip_description(iso_code, slug):
    try:
        return Tooltip.objects.get(status=1, slug=slug, language__code__iexact=iso_code).description
    except Exception:
        return""


class ExtendedURLNode(defaulttags.URLNode):
    def __init__(self, view_name, args, kwargs, asvar, site_domain):
            self.view_name = view_name
            self.args = args
            self.kwargs = kwargs
            self.asvar = asvar
            self.site_domain = site_domain

    def render(self, context):
        try:
            site_domain = "http://%s" % Site.objects.get(name=self.site_domain).domain
        except Exception:
            site_domain=""

        retval = super(ExtendedURLNode, self).render(context)
        retval = site_domain + retval
        if self.asvar:
            context[self.asvar] = retval
            return ''
        else:
            return retval

@register.tag
def ext_url(parser, token,):
    site_domain = token.split_contents()[len(token.split_contents())-1]
    token.contents = " ".join(token.split_contents()[:len(token.split_contents())-1])
    node_instance = defaulttags.url(parser, token)
    used_class = ExtendedURLNode
    return used_class(view_name=node_instance.view_name,
                      args=node_instance.args,
                      kwargs=node_instance.kwargs,
                      asvar=node_instance.asvar,
                      site_domain=site_domain)


@register.filter(name='is_media_photo')
def is_media_photo(formset):
    from publieditor.content.models import MediaPhoto
    return formset.opts.model == MediaPhoto

@register.filter(name='is_media_video')
def is_media_video(formset):
    from publieditor.content.models import MediaVideo
    return formset.opts.model == MediaVideo

@register.filter(name='is_media_table')
def is_media_table(formset):
    from publieditor.content.models import MediaTable
    return formset.opts.model == MediaTable